@@ -32,7 +32,9 @@
 #include <c-strcase.h>
 #include <c-ctype.h>
 #include <auth/pam.h>
+#include <auth/radius.h>
 #include <auth/plain.h>
+#include <sec-mod-sup-config.h>
 
 #include <vpn.h>
 #include <cookies.h>
@@ -45,6 +47,7 @@
 
 static char pid_file[_POSIX_PATH_MAX] = "";
 static const char* cfg_file = DEFAULT_CFG_FILE;
+static const struct auth_mod_st *amod = NULL;
 
 struct cfg_options {
 	const char* name;
@@ -56,11 +59,14 @@ struct cfg_options {
 static struct cfg_options available_options[] = {
 	{ .name = "auth", .type = OPTION_MULTI_LINE, .mandatory = 1 },
 	{ .name = "route", .type = OPTION_MULTI_LINE, .mandatory = 0 },
+	{ .name = "no-route", .type = OPTION_MULTI_LINE, .mandatory = 0 },
 	{ .name = "select-group", .type = OPTION_MULTI_LINE, .mandatory = 0 },
 	{ .name = "custom-header", .type = OPTION_MULTI_LINE, .mandatory = 0 },
 	{ .name = "split-dns", .type = OPTION_MULTI_LINE, .mandatory = 0 },
 	{ .name = "listen-host", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "listen-host-is-dyndns", .type = OPTION_BOOLEAN, .mandatory = 0 },
+	{ .name = "compression", .type = OPTION_BOOLEAN, .mandatory = 0 },
+	{ .name = "no-compress-limit", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "tcp-port", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "udp-port", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "keepalive", .type = OPTION_NUMERIC, .mandatory = 0 },
@@ -87,6 +93,7 @@ static struct cfg_options available_options[] = {
 	{ .name = "occtl-socket-file", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "banner", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "use-seccomp", .type = OPTION_BOOLEAN, .mandatory = 0 },
+	{ .name = "isolate-workers", .type = OPTION_BOOLEAN, .mandatory = 0 },
 	{ .name = "predictable-ips", .type = OPTION_BOOLEAN, .mandatory = 0 },
 	{ .name = "session-control", .type = OPTION_BOOLEAN, .mandatory = 0 },
 	{ .name = "auto-select-group", .type = OPTION_BOOLEAN, .mandatory = 0 },
@@ -106,6 +113,7 @@ static struct cfg_options available_options[] = {
 	{ .name = "net-priority", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "output-buffer", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "cookie-timeout", .type = OPTION_NUMERIC, .mandatory = 0 },
+	{ .name = "stats-report-time", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "rekey-time", .type = OPTION_NUMERIC, .mandatory = 0 },
 	{ .name = "rekey-method", .type = OPTION_STRING, .mandatory = 0 },
 	{ .name = "auth-timeout", .type = OPTION_NUMERIC, .mandatory = 0 },
@@ -143,7 +151,44 @@ static struct cfg_options available_options[] = {
 	{ .name = "default-group-config", .type = OPTION_STRING, .mandatory = 0 },
 };
 
-static char *get_brackets_string(void *pool, const char *str);
+#define get_brackets_string get_brackets_string1
+static char *get_brackets_string1(void *pool, const char *str);
+
+#ifdef HAVE_RADIUS
+static char *get_brackets_string2(void *pool, const char *str)
+{
+	char *p, *p2;
+	unsigned len;
+
+	p = strchr(str, '[');
+	if (p == NULL) {
+		return NULL;
+	}
+	p++;
+
+	p = strchr(p, ',');
+	if (p == NULL) {
+		return NULL;
+	}
+	p++;
+
+	while (c_isspace(*p))
+		p++;
+
+	p2 = strchr(p, ',');
+	if (p2 == NULL) {
+		p2 = strchr(p, ']');
+		if (p2 == NULL) {
+			fprintf(stderr, "error parsing %s\n", str);
+			exit(1);
+		}
+	}
+
+	len = p2 - p;
+
+	return talloc_strndup(pool, p, len);
+}
+#endif
 
 static const tOptionValue* get_option(const char* name, unsigned * mand)
 {
@@ -227,7 +272,7 @@ unsigned j;
 #define READ_STATIC_STRING(name, s_name) { \
 	val = get_option(name, &mand); \
 	if (val != NULL && val->valType == OPARG_TYPE_STRING) \
-		snprintf(s_name, sizeof(s_name), "%s", val->v.strVal); \
+		strlcpy(s_name, val->v.strVal, sizeof(s_name)); \
 	else if (mand != 0) { \
 		fprintf(stderr, "Configuration option %s is mandatory.\n", name); \
 		exit(1); \
@@ -300,7 +345,7 @@ unsigned j;
 	}
 }
 
-static char *get_brackets_string(void *pool, const char *str)
+static char *get_brackets_string1(void *pool, const char *str)
 {
 	char *p, *p2;
 	unsigned len;
@@ -313,10 +358,13 @@ static char *get_brackets_string(void *pool, const char *str)
 	while (c_isspace(*p))
 		p++;
 
-	p2 = strchr(p, ']');
+	p2 = strchr(p, ',');
 	if (p2 == NULL) {
-		fprintf(stderr, "error parsing %s\n", str);
-		exit(1);
+		p2 = strchr(p, ']');
+		if (p2 == NULL) {
+			fprintf(stderr, "error parsing %s\n", str);
+			exit(1);
+		}
 	}
 
 	len = p2 - p;
@@ -324,6 +372,34 @@ static char *get_brackets_string(void *pool, const char *str)
 	return talloc_strndup(pool, p, len);
 }
 
+
+/* Parses the string ::1/prefix, to return prefix
+ * and modify the string to contain the network only.
+ */
+unsigned extract_prefix(char *network)
+{
+	char *p;
+	unsigned prefix;
+
+	if (network == NULL)
+		return 0;
+
+	p = strchr(network, '/');
+
+	if (p == NULL)
+		return 0;
+
+	prefix = atoi(p+1);
+	*p = 0;
+
+	return prefix;
+}
+
+const struct auth_mod_st *get_auth_mod(void)
+{
+	return amod;
+}
+
 static void parse_cfg_file(const char* file, struct cfg_st *config, unsigned reload)
 {
 tOptionValue const * pov;
@@ -332,7 +408,7 @@ unsigned j, i, mand;
 char** auth = NULL;
 unsigned auth_size = 0;
 unsigned prefix = 0, auto_select_group = 0;
-const struct auth_mod_st *amod = NULL;
+unsigned prefix4 = 0;
 char *tmp;
 unsigned force_cert_auth;
 
@@ -360,6 +436,8 @@ unsigned force_cert_auth;
 		prev = val;
 	} while((val = optionNextValue(pov, prev)) != NULL);
 
+	config->sup_config_type = SUP_CONFIG_FILE;
+
 	READ_MULTI_LINE("auth", auth, auth_size);
 	for (j=0;j<auth_size;j++) {
 		if (c_strncasecmp(auth[j], "pam", 3) == 0) {
@@ -369,8 +447,8 @@ unsigned force_cert_auth;
 				exit(1);
 			}
 #ifdef HAVE_PAM
-			config->auth_types |= AUTH_TYPE_PAM;
 			amod = &pam_auth_funcs;
+			config->auth_types |= amod->type;
 #else
 			fprintf(stderr, "PAM support is disabled\n");
 			exit(1);
@@ -387,11 +465,42 @@ unsigned force_cert_auth;
 				exit(1);
 			}
 			amod = &plain_auth_funcs;
-			config->auth_types |= AUTH_TYPE_PLAIN;
+			config->auth_types |= amod->type;
+		} else if (strncasecmp(auth[j], "radius", 6) == 0) {
+			if ((config->auth_types & AUTH_TYPE_USERNAME_PASS) != 0) {
+				fprintf(stderr, "You cannot mix multiple username/password authentication methods\n");
+				exit(1);
+			} else {
+
+#ifdef HAVE_RADIUS
+				const char *p;
+
+				config->auth_additional = get_brackets_string1(config, auth[j]+6);
+				if (config->auth_additional == NULL) {
+					fprintf(stderr, "No configuration specified; error in %s\n", auth[j]);
+					exit(1);
+				}
+
+				p = get_brackets_string2(config, auth[j]+6);
+				if (p != NULL) {
+					if (strcasecmp(p, "groupconfig") != 0) {
+						fprintf(stderr, "No known configuration option: %s\n", p);
+						exit(1);
+					}
+					config->sup_config_type = SUP_CONFIG_RADIUS;
+				}
+				amod = &radius_auth_funcs;
+				config->auth_types |= amod->type;
+#else
+				fprintf(stderr, "Radius support is disabled\n");
+				exit(1);
+#endif
+			}
 		} else if (c_strcasecmp(auth[j], "certificate") == 0) {
 			config->auth_types |= AUTH_TYPE_CERTIFICATE;
 		} else if (c_strcasecmp(auth[j], "certificate[optional]") == 0) {
 			config->auth_types |= AUTH_TYPE_CERTIFICATE_OPT;
+			fprintf(stderr, "The authentication option certificate[optional] is experimental and may be removed in the future\n");
 		} else {
 			fprintf(stderr, "Unknown auth method: %s\n", auth[j]);
 			exit(1);
@@ -452,8 +561,9 @@ unsigned force_cert_auth;
 	if (config->occtl_socket_file == NULL)
 		config->occtl_socket_file = talloc_strdup(config, OCCTL_UNIX_SOCKET);
 
-	if (config->auth_types & AUTH_TYPE_USERNAME_PASS) {
-		READ_TF("session-control", config->session_control, 0);
+	val = get_option("session-control", NULL);
+	if (val != NULL) {
+		fprintf(stderr, "The option 'session-control' is deprecated\n");
 	}
 
 	READ_STRING("banner", config->banner);
@@ -464,7 +574,26 @@ unsigned force_cert_auth;
 		config->cisco_client_compat = 1;
 	}
 
-	READ_TF("use-seccomp", config->seccomp, 0);
+	READ_TF("compression", config->enable_compression, 0);
+	READ_NUMERIC("no-compress-limit", config->no_compress_limit);
+	if (config->no_compress_limit == 0)
+		config->no_compress_limit = DEFAULT_NO_COMPRESS_LIMIT;
+	if (config->no_compress_limit < MIN_NO_COMPRESS_LIMIT)
+		config->no_compress_limit = MIN_NO_COMPRESS_LIMIT;
+
+	READ_TF("use-seccomp", config->isolate, 0);
+	if (config->isolate) {
+		fprintf(stderr, "note that 'use-seccomp' was replaced by 'isolate-workers'\n");
+	} else {
+		READ_TF("isolate-workers", config->isolate, 0);
+	}
+#if !defined(HAVE_LIBSECCOMP) && !defined(ENABLE_LINUX_NS)
+	if (config->isolate != 0) {
+		fprintf(stderr, "error: 'isolate-workers' is set to true, but not compiled with seccomp or Linux namespaces support\n");
+		exit(1);
+	}
+#endif
+
 	READ_TF("predictable-ips", config->predictable_ips, 1);
 	READ_TF("use-utmp", config->use_utmp, 1);
 	READ_TF("use-dbus", config->use_dbus, 0);
@@ -496,6 +625,8 @@ unsigned force_cert_auth;
 
 	READ_TF("deny-roaming", config->deny_roaming, 0);
 
+	READ_NUMERIC("stats-report-time", config->stats_report_time);
+
 	config->rekey_time = -1;
 	READ_NUMERIC("rekey-time", config->rekey_time);
 	if (config->rekey_time == -1) {
@@ -555,16 +686,25 @@ unsigned force_cert_auth;
 	READ_STRING("proxy-url", config->proxy_url);
 
 	READ_STRING("ipv4-network", config->network.ipv4);
-	READ_STRING("ipv4-netmask", config->network.ipv4_netmask);
+
+	prefix4 = extract_prefix(config->network.ipv4);
+	if (prefix4 == 0) {
+		READ_STRING("ipv4-netmask", config->network.ipv4_netmask);
+	} else {
+		config->network.ipv4_netmask = ipv4_prefix_to_mask(config, prefix4);
+	}
 
 	READ_STRING("ipv6-network", config->network.ipv6);
 
-	READ_NUMERIC("ipv6-prefix", prefix);
+	prefix = extract_prefix(config->network.ipv6);
+	if (prefix == 0) {
+		READ_NUMERIC("ipv6-prefix", prefix);
+	}
+
 	if (prefix > 0) {
-		config->network.ipv6_netmask = ipv6_prefix_to_mask(config, prefix);
 		config->network.ipv6_prefix = prefix;
 
-		if (config->network.ipv6_netmask == NULL) {
+		if (valid_ipv6_prefix(prefix) == 0) {
 			fprintf(stderr, "invalid IPv6 prefix: %u\n", prefix);
 			exit(1);
 		}
@@ -579,12 +719,14 @@ unsigned force_cert_auth;
 		    strcmp(config->network.routes[j], "default") == 0) {
 		    	/* set default route */
 			for (i=0;i<j;i++)
-				free(config->network.routes[i]);
+				talloc_free(config->network.routes[i]);
 			config->network.routes_size = 0;
 			break;
 		}
 	}
 
+	READ_MULTI_LINE("no-route", config->network.no_routes, config->network.no_routes_size);
+
 	READ_STRING("default-select-group", config->default_select_group);
 	READ_TF("auto-select-group", auto_select_group, 0);
 	if (auto_select_group != 0 && amod != NULL && amod->group_list != NULL) {
@@ -642,8 +784,8 @@ static void check_cfg(struct cfg_st *config)
 		exit(1);
 	}
 
-	if (config->network.ipv6 != NULL && config->network.ipv6_netmask == NULL) {
-		fprintf(stderr, "No mask found for IPv6 network.\n");
+	if (config->network.ipv6 != NULL && config->network.ipv6_prefix == 0) {
+		fprintf(stderr, "No prefix found for IPv6 network.\n");
 		exit(1);
 	}
 
@@ -717,7 +859,7 @@ int cmd_parser (void *pool, int argc, char **argv, struct cfg_st** config)
 		(*config)->foreground = 1;
 
 	if (HAVE_OPT(PID_FILE)) {
-		snprintf(pid_file, sizeof(pid_file), "%s", OPT_ARG(PID_FILE));
+		strlcpy(pid_file, OPT_ARG(PID_FILE), sizeof(pid_file));
 	}
 
 	if (HAVE_OPT(DEBUG))
@@ -775,7 +917,6 @@ unsigned i;
 	DEL(config->network.ipv4);
 	DEL(config->network.ipv4_netmask);
 	DEL(config->network.ipv6);
-	DEL(config->network.ipv6_netmask);
 	for (i=0;i<config->network.routes_size;i++)
 		DEL(config->network.routes[i]);
 	DEL(config->network.routes);
@@ -819,9 +960,15 @@ void print_version(tOptions *opts, tOptDesc *desc)
 #ifdef HAVE_LIBSECCOMP
 	fprintf(stderr, "seccomp, ");
 #endif
+#ifdef ENABLE_LINUX_NS
+	fprintf(stderr, "Linux-NS, ");
+#endif
 #ifdef HAVE_LIBWRAP
 	fprintf(stderr, "tcp-wrappers, ");
 #endif
+#ifdef HAVE_RADIUS
+	fprintf(stderr, "radius, ");
+#endif
 #ifdef HAVE_PAM
 	fprintf(stderr, "PAM, ");
 #endif
@@ -846,6 +993,7 @@ void print_version(tOptions *opts, tOptDesc *desc)
 void reload_cfg_file(void *pool, struct cfg_st* config)
 {
 	clear_cfg_file(config);
+
 	memset(config, 0, sizeof(*config));
 
 	parse_cfg_file(cfg_file, config, 1);
