@@ -172,14 +172,12 @@ ssize_t recvfrom_timeout(int sockfd, void *buf, size_t len, int flags,
 
 }
 
-int icmp_ping4(main_server_st * s, struct sockaddr_in *addr1,
-	       struct sockaddr_in *addr2)
+int icmp_ping4(main_server_st * s, struct sockaddr_in *addr1)
 {
 	struct icmp *pkt;
 	int pingsock, c, e;
 	char packet1[DEFDATALEN + MAXIPLEN + MAXICMPLEN];
-	char packet2[DEFDATALEN + MAXIPLEN + MAXICMPLEN];
-	char buf1[64], buf2[64];
+	char buf1[64];
 	time_t now;
 	uint16_t id1, id2;
 	unsigned gotreply = 0, unreachable = 0;
@@ -205,20 +203,9 @@ int icmp_ping4(main_server_st * s, struct sockaddr_in *addr1,
 	pkt->icmp_cksum =
 	    in_cksum((unsigned short *) pkt, sizeof(packet1));
 
-	pkt = (struct icmp *) packet2;
-	memset(pkt, 0, sizeof(packet2));
-	pkt->icmp_type = ICMP_ECHO;
-	pkt->icmp_id = id2;
-	pkt->icmp_cksum =
-	    in_cksum((unsigned short *) pkt, sizeof(packet2));
-
 	while ((c = sendto(pingsock, packet1, DEFDATALEN + ICMP_MINLEN, 0,
 			   (struct sockaddr *) addr1,
 			   sizeof(*addr1)) == -1) && retry(errno));
-	while ((c =
-		sendto(pingsock, packet2, DEFDATALEN + ICMP_MINLEN, 0,
-		       (struct sockaddr *) addr2, sizeof(*addr2)) == -1)
-	       && retry(errno));
 	/* listen for replies */
 
 	now = time(0);
@@ -235,10 +222,7 @@ int icmp_ping4(main_server_st * s, struct sockaddr_in *addr1,
 		} else if (c >= 76 && fromlen == sizeof(struct sockaddr_in)) {	/* icmp6_hdr */
 			if (memcmp
 			    (SA_IN_P(&from), SA_IN_P(addr1),
-			     SA_IN_SIZE(sizeof(*addr1))) == 0 ||
-			    memcmp
-			    (SA_IN_P(&from), SA_IN_P(addr2),
-			     SA_IN_SIZE(sizeof(*addr2))) == 0) {
+			     SA_IN_SIZE(sizeof(*addr1))) == 0) {
 
 #ifdef HAVE_STRUCT_IPHDR_IHL
 				struct iphdr *iphdr =
@@ -261,38 +245,29 @@ int icmp_ping4(main_server_st * s, struct sockaddr_in *addr1,
 
 	if (gotreply > 0) {
 		mslog(s, NULL, LOG_INFO,
-		      "pinged %s and %s and are in use",
+		      "pinged %s and is in use",
 		      human_addr((void *) addr1,
 				 sizeof(struct sockaddr_in), buf1,
-				 sizeof(buf1)), human_addr((void *) addr2,
-							   sizeof(struct
-								  sockaddr_in),
-							   buf2,
-							   sizeof(buf2)));
+				 sizeof(buf1)));
 		return gotreply;
 	} else {
 		mslog(s, NULL, LOG_INFO,
-		      "pinged %s and %s and are not in use",
+		      "pinged %s and is not in use",
 		      human_addr((void *) addr1,
 				 sizeof(struct sockaddr_in), buf1,
-				 sizeof(buf1)), human_addr((void *) addr2,
-							   sizeof(struct
-								  sockaddr_in),
-							   buf2,
-							   sizeof(buf2)));
+				 sizeof(buf1)));
 		return 0;
 	}
 }
 
 int icmp_ping6(main_server_st * s,
-	       struct sockaddr_in6 *addr1, struct sockaddr_in6 *addr2)
+	       struct sockaddr_in6 *addr1)
 {
 	struct icmp6_hdr *pkt;
-	char buf1[64], buf2[64];
+	char buf1[64];
 	int pingsock, c, e;
 	int sockopt;
 	char packet1[DEFDATALEN + MAXIPLEN + MAXICMPLEN];
-	char packet2[DEFDATALEN + MAXIPLEN + MAXICMPLEN];
 	uint16_t id1, id2;
 	unsigned gotreply = 0, unreachable = 0;
 	time_t now;
@@ -316,11 +291,6 @@ int icmp_ping6(main_server_st * s,
 	pkt->icmp6_type = ICMP6_ECHO_REQUEST;
 	pkt->icmp6_id = id1;
 
-	pkt = (struct icmp6_hdr *) packet2;
-	memset(pkt, 0, sizeof(packet2));
-	pkt->icmp6_type = ICMP6_ECHO_REQUEST;
-	pkt->icmp6_id = id2;
-
 #if defined(SOL_RAW) && defined(IPV6_CHECKSUM)
 	sockopt = offsetof(struct icmp6_hdr, icmp6_cksum);
 	setsockopt(pingsock, SOL_RAW, IPV6_CHECKSUM,
@@ -332,13 +302,6 @@ int icmp_ping6(main_server_st * s,
 		       (struct sockaddr *) addr1,
 		       sizeof(*addr1)) == -1) && retry(errno));
 
-	while ((c =
-		sendto(pingsock, packet2,
-		       DEFDATALEN +
-		       sizeof(struct icmp6_hdr), 0,
-		       (struct sockaddr *) addr2, sizeof(*addr2)) == -1)
-	       && retry(errno));
-
 	/* listen for replies */
 	now = time(0);
 	while (time(0) - now < PING_TIMEOUT
@@ -354,10 +317,7 @@ int icmp_ping6(main_server_st * s,
 		} else if (c >= 8 && fromlen == sizeof(struct sockaddr_in6)) {	/* icmp6_hdr */
 			if (memcmp
 			    (SA_IN6_P(&from), SA_IN6_P(addr1),
-			     SA_IN_SIZE(sizeof(*addr1))) == 0 ||
-			    memcmp(SA_IN6_P(&from),
-				   SA_IN6_P(addr2),
-				   SA_IN_SIZE(sizeof(*addr2))) == 0) {
+			     SA_IN_SIZE(sizeof(*addr1))) == 0) {
 
 				pkt = (struct icmp6_hdr *) packet1;
 				if (pkt->icmp6_id == id1 || pkt->icmp6_id == id2) {
@@ -374,25 +334,17 @@ int icmp_ping6(main_server_st * s,
 
 	if (gotreply > 0) {
 		mslog(s, NULL, LOG_INFO,
-		      "pinged %s and %s and are in use",
+		      "pinged %s and is in use",
 		      human_addr((void *) addr1,
 				 sizeof(struct sockaddr_in6), buf1,
-				 sizeof(buf1)), human_addr((void *) addr2,
-							   sizeof(struct
-								  sockaddr_in6),
-							   buf2,
-							   sizeof(buf2)));
+				 sizeof(buf1)));
 		return gotreply;
 	} else {
 		mslog(s, NULL, LOG_INFO,
-		      "pinged %s and %s and are not in use",
+		      "pinged %s and is not in use",
 		      human_addr((void *) addr1,
 				 sizeof(struct sockaddr_in6), buf1,
-				 sizeof(buf1)), human_addr((void *) addr2,
-							   sizeof(struct
-								  sockaddr_in6),
-							   buf2,
-							   sizeof(buf2)));
+				 sizeof(buf1)));
 		return 0;
 	}
 }
