@@ -28,6 +28,7 @@
 #include <vpn.h>
 #include <worker.h>
 #include <main.h>
+#include <sec-mod.h>
 
 char *human_addr2(const struct sockaddr *sa, socklen_t salen,
 		       void *_buf, size_t buflen, unsigned full)
@@ -95,7 +96,7 @@ void __attribute__ ((format(printf, 3, 4)))
 	const char* ip;
 	va_list args;
 
-	if (priority == LOG_DEBUG && ws->config->debug == 0)
+	if (priority == LOG_DEBUG && ws->config->debug < 3)
 		return;
 
 	if (priority == LOG_HTTP_DEBUG) {
@@ -139,7 +140,7 @@ void __attribute__ ((format(printf, 4, 5)))
 	const char* ip = NULL;
 	va_list args;
 
-	if (priority == LOG_DEBUG && s->config->debug == 0)
+	if (priority == LOG_DEBUG && s->config->debug < 3)
 		return;
 
 	if (priority == LOG_HTTP_DEBUG) {
@@ -224,3 +225,28 @@ void  oclog_hex(const worker_st* ws, int priority,
 
 	return;
 }
+
+void  seclog_hex(const struct sec_mod_st* sec, int priority,
+		const char *prefix, uint8_t* bin, unsigned bin_size, unsigned b64)
+{
+	char buf[512];
+	int ret;
+	size_t buf_size;
+	gnutls_datum_t data = {bin, bin_size};
+
+	if (priority == LOG_DEBUG && sec->config->debug == 0)
+		return;
+
+	if (b64) {
+		base64_encode((char*)bin, bin_size, (char*)buf, sizeof(buf));
+	} else {
+		buf_size = sizeof(buf);
+		ret = gnutls_hex_encode(&data, buf, &buf_size);
+		if (ret < 0)
+			return;
+	}
+
+	seclog(sec, priority, "%s %s", prefix, buf);
+
+	return;
+}
