@@ -36,7 +36,6 @@
 #include <script-list.h>
 #include <ip-lease.h>
 #include <proc-search.h>
-#include <main-sup-config.h>
 #include "str.h"
 
 #include <vpn.h>
@@ -85,7 +84,6 @@ int send_cookie_auth_reply(main_server_st* s, struct proc_st* proc,
 		}
 
 		msg.ipv4_netmask = proc->config.ipv4_netmask;
-		msg.ipv6_netmask = proc->config.ipv6_netmask;
 
 		msg.ipv4_network = proc->config.ipv4_network;
 		msg.ipv6_network = proc->config.ipv6_network;
@@ -133,6 +131,12 @@ int send_cookie_auth_reply(main_server_st* s, struct proc_st* proc,
 			msg.routes = proc->config.routes;
 		}
 
+		msg.n_no_routes = proc->config.no_routes_size;
+		for (i=0;i<proc->config.no_routes_size;i++) {
+			mslog(s, proc, LOG_DEBUG, "sending no-route '%s'", proc->config.no_routes[i]);
+			msg.no_routes = proc->config.no_routes;
+		}
+
 		ret = send_socket_msg_to_worker(s, proc, AUTH_COOKIE_REP, proc->tun_lease.fd,
 			 &msg,
 			 (pack_size_func)auth_reply_msg__get_packed_size,
@@ -166,11 +170,10 @@ int handle_auth_cookie_req(main_server_st* s, struct proc_st* proc,
 {
 int ret;
 Cookie *cmsg;
-time_t now = time(0);
 gnutls_datum_t key = {s->cookie_key, sizeof(s->cookie_key)};
 char str_ip[MAX_IP_STR+1];
 PROTOBUF_ALLOCATOR(pa, proc);
-struct cookie_entry_st *old;
+struct proc_st *old_proc;
 
 	if (req->cookie.len == 0) {
 		mslog(s, proc, LOG_INFO, "error in cookie size");
@@ -185,7 +188,7 @@ struct cookie_entry_st *old;
 
 	if (cmsg->username == NULL)
 		return -1;
-	snprintf(proc->username, sizeof(proc->username), "%s", cmsg->username);
+	strlcpy(proc->username, cmsg->username, sizeof(proc->username));
 
 	if (cmsg->sid.len != sizeof(proc->sid))
 		return -1;
@@ -203,24 +206,23 @@ struct cookie_entry_st *old;
 	/* override the group name in order to load the correct configuration in
 	 * case his group is specified in the certificate */
 	if (cmsg->groupname)
-		snprintf(proc->groupname, sizeof(proc->groupname), "%s", cmsg->groupname);
+		strlcpy(proc->groupname, cmsg->groupname, sizeof(proc->groupname));
 
 	/* cookie is good so far, now read config (in order to know
 	 * whether roaming is allowed or not */
 	memset(&proc->config, 0, sizeof(proc->config));
 	apply_default_sup_config(s->config, proc);
 
-	if (s->config_module) {
-		ret = s->config_module->get_sup_config(s->config, proc);
-		if (ret < 0) {
-			mslog(s, proc, LOG_ERR,
-			      "error reading additional configuration");
-			return ERR_READ_CONFIG;
-		}
+	/* loads sup config */
+	ret = session_open(s, proc, req->cookie.data, req->cookie.len);
+	if (ret < 0) {
+		mslog(s, proc, LOG_INFO, "could not open session");
+		return -1;
+	}
 
-	        if (proc->config.cgroup != NULL) {
-	        	put_into_cgroup(s, proc->config.cgroup, proc->pid);
-		}
+	/* Put into right cgroup */
+        if (proc->config.cgroup != NULL) {
+        	put_into_cgroup(s, proc->config.cgroup, proc->pid);
 	}
 
 	/* check whether the cookie IP matches */
@@ -240,37 +242,25 @@ struct cookie_entry_st *old;
 		}
 	}
 
-	/* check for a valid stored cookie */
-	if ((old=find_cookie_entry(&s->cookies, req->cookie.data, req->cookie.len)) != NULL) {
-		mslog(s, proc, LOG_DEBUG, "reusing cookie for '%s' (%u)", proc->username, (unsigned)proc->pid);
-		if (old->proc != NULL) {
-			mslog(s, old->proc, LOG_DEBUG, "disconnecting '%s' (%u) due to new cookie connection",
-				old->proc->username, (unsigned)old->proc->pid);
+	/* check for a user with the same sid as in the cookie */
+	old_proc = proc_search_sid(s, cmsg->sid.data);
+	if (old_proc != NULL) {
+		mslog(s, old_proc, LOG_DEBUG, "disconnecting (%u) due to new cookie session",
+			(unsigned)old_proc->pid);
 
-			/* steal its leases */
-			steal_ip_leases(old->proc, proc);
-
-			/* steal its cookie */
-			old->proc->cookie_ptr = NULL;
-
-			if (old->proc->pid > 0)
-				kill(old->proc->pid, SIGTERM);
-		} else {
-			revive_cookie(old);
+		if (strcmp(proc->username, old_proc->username) != 0) {
+			mslog(s, old_proc, LOG_ERR, "the user of the cookie doesn't match (new: %s)",
+				proc->username);
+			return -1;
 		}
-		proc->cookie_ptr = old;
-		old->proc = proc;
+
+		/* steal its leases */
+		steal_ip_leases(old_proc, proc);
+
+		if (old_proc->pid > 0)
+			kill(old_proc->pid, SIGTERM);
 	} else {
-		if (cmsg->expiration < now) {
-			mslog(s, proc, LOG_INFO, "ignoring expired cookie");
-			return -1;
-		}
-
-		mslog(s, proc, LOG_DEBUG, "new cookie for '%s' (%u)", proc->username, (unsigned)proc->pid);
-
-		proc->cookie_ptr = new_cookie_entry(&s->cookies, proc, req->cookie.data, req->cookie.len);
-		if (proc->cookie_ptr == NULL)
-			return -1;
+		mslog(s, proc, LOG_DEBUG, "new cookie session for (%u)", (unsigned)proc->pid);
 	}
 
 	if (proc->config.require_cert != 0 && cmsg->tls_auth_ok == 0) {
@@ -280,19 +270,16 @@ struct cookie_entry_st *old;
 	}
 
 	if (cmsg->hostname)
-		snprintf(proc->hostname, sizeof(proc->hostname), "%s", cmsg->hostname);
+		strlcpy(proc->hostname, cmsg->hostname, sizeof(proc->hostname));
 
 	memcpy(proc->ipv4_seed, &cmsg->ipv4_seed, sizeof(proc->ipv4_seed));
 
-	ret = session_open(s, proc);
-	if (ret < 0) {
-		mslog(s, proc, LOG_INFO, "could not open session");
+	/* add the links to proc hash */
+	if (proc_table_add(s, proc) < 0) {
+		mslog(s, proc, LOG_ERR, "failed to add proc hashes");
 		return -1;
 	}
 
-	/* add the links to proc hash */
-	proc_table_add(s, proc);
-
 	return 0;
 }
 
