@@ -32,7 +32,7 @@
 #include <sys/un.h>
 #include <sys/uio.h>
 
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__OpenBSD__)
 # include <limits.h>
 # define SOL_IP IPPROTO_IP
 #endif
@@ -80,19 +80,6 @@ enum {
 	PS_AUTH_COMPLETED, /* successful authentication */
 };
 
-#define COOKIE_HASH_SIZE 20
-#define COOKIE_HASH GNUTLS_DIG_SHA1
-
-typedef struct cookie_entry_st {
-	struct proc_st *proc; /* may be null, otherwise the proc that uses that cookie */
-	time_t expiration; /* -1 or the time it should expire */
-
-	/* We store the hash of the cookie that is associated with a particular session.
-	 * The reason is to avoid a memory leak to an unprivileged process to expose
-	 * data that can be used to authenticate as another use */
-	uint8_t cookie_hash[COOKIE_HASH_SIZE];
-} cookie_st;
-
 /* Each worker process maps to a unique proc_st structure.
  */
 typedef struct proc_st {
@@ -132,9 +119,9 @@ typedef struct proc_st {
 	char user_agent[MAX_AGENT_NAME];
 	char tls_ciphersuite[MAX_CIPHERSUITE_NAME];
 	char dtls_ciphersuite[MAX_CIPHERSUITE_NAME];
-
-	/* pointer to the cookie used by this session */
-	struct cookie_entry_st *cookie_ptr;
+	char cstp_compr[8];
+	char dtls_compr[8];
+	unsigned mtu;
 
 	/* if the session is initiated by a cookie the following two are set
 	 * and are considered when generating an IP address. That is used to
@@ -181,6 +168,7 @@ struct cookie_entry_db_st {
 
 struct proc_hash_db_st {
 	struct htable *db_ip;
+	struct htable *db_dtls_id;
 	struct htable *db_sid;
 	unsigned total;
 };
@@ -213,7 +201,6 @@ typedef struct main_server_st {
 	time_t start_time;
 
 	void * auth_extra;
-	struct config_mod_st *config_module;
 
 #ifdef HAVE_DBUS
 	void * ctl_ctx;
@@ -244,7 +231,7 @@ int handle_resume_fetch_req(main_server_st* s, struct proc_st * proc,
 int handle_resume_store_req(main_server_st* s, struct proc_st *proc,
   			   const SessionResumeStoreReqMsg *);
 
-int session_open(main_server_st * s, struct proc_st *proc);
+int session_open(main_server_st * s, struct proc_st *proc, const uint8_t *cookie, unsigned cookie_size);
 int session_close(main_server_st * s, struct proc_st *proc);
 
 void 
@@ -305,4 +292,6 @@ int send_socket_msg_to_worker(main_server_st* s, struct proc_st* proc, uint8_t c
 void request_reload(int signo);
 void request_stop(int signo);
 
+const struct auth_mod_st *get_auth_mod(void);
+
 #endif
