@@ -49,19 +49,26 @@
 #include <auth/pam.h>
 #include <sec-mod.h>
 #include <vpn.h>
+#include <sec-mod-sup-config.h>
 
 static const struct auth_mod_st *module = NULL;
 
-void sec_auth_init(struct cfg_st *config)
+void sec_auth_init(sec_mod_st * sec, struct cfg_st *config)
 {
-#ifdef HAVE_PAM
-	if ((config->auth_types & pam_auth_funcs.type) == pam_auth_funcs.type)
-		module = &pam_auth_funcs;
-	else
-#endif
-	if ((config->auth_types & plain_auth_funcs.type) ==
-		    plain_auth_funcs.type) {
-		module = &plain_auth_funcs;
+	module = get_auth_mod();
+
+	if (module && module->global_init) {
+		module->global_init(sec, config->auth_additional);
+	}
+}
+
+void sec_auth_reinit(sec_mod_st * sec, struct cfg_st *config)
+{
+	if (module) {
+		if (module != get_auth_mod()) {
+			seclog(sec, LOG_ERR, "Cannot change authentication method on reload");
+			exit(1);
+		}
 	}
 }
 
@@ -102,7 +109,7 @@ static int generate_cookie(sec_mod_st * sec, client_entry_st * entry)
 }
 
 static
-int send_sec_auth_reply(sec_mod_st * sec, client_entry_st * entry, AUTHREP r)
+int send_sec_auth_reply(int cfd, sec_mod_st * sec, client_entry_st * entry, AUTHREP r)
 {
 	SecAuthReplyMsg msg = SEC_AUTH_REPLY_MSG__INIT;
 	int ret;
@@ -130,7 +137,7 @@ int send_sec_auth_reply(sec_mod_st * sec, client_entry_st * entry, AUTHREP r)
 		msg.dtls_session_id.data = entry->dtls_session_id;
 		msg.dtls_session_id.len = sizeof(entry->dtls_session_id);
 
-		ret = send_msg(entry, sec->fd, SM_CMD_AUTH_REP,
+		ret = send_msg(entry, cfd, SM_CMD_AUTH_REP,
 			       &msg,
 			       (pack_size_func)
 			       sec_auth_reply_msg__get_packed_size,
@@ -138,7 +145,7 @@ int send_sec_auth_reply(sec_mod_st * sec, client_entry_st * entry, AUTHREP r)
 	} else {
 		msg.reply = AUTH__REP__FAILED;
 
-		ret = send_msg(entry, sec->fd, SM_CMD_AUTH_REP,
+		ret = send_msg(entry, cfd, SM_CMD_AUTH_REP,
 			       &msg,
 			       (pack_size_func)
 			       sec_auth_reply_msg__get_packed_size,
@@ -155,7 +162,7 @@ int send_sec_auth_reply(sec_mod_st * sec, client_entry_st * entry, AUTHREP r)
 }
 
 static
-int send_sec_auth_reply_msg(sec_mod_st * sec, client_entry_st * e)
+int send_sec_auth_reply_msg(int cfd, sec_mod_st * sec, client_entry_st * e)
 {
 	SecAuthReplyMsg msg = SEC_AUTH_REPLY_MSG__INIT;
 	char tmp[MAX_MSG_SIZE] = "";
@@ -176,7 +183,7 @@ int send_sec_auth_reply_msg(sec_mod_st * sec, client_entry_st * e)
 	msg.sid.data = e->sid;
 	msg.sid.len = sizeof(e->sid);
 
-	ret = send_msg(e, sec->fd, SM_CMD_AUTH_REP, &msg,
+	ret = send_msg(e, cfd, SM_CMD_AUTH_REP, &msg,
 		       (pack_size_func) sec_auth_reply_msg__get_packed_size,
 		       (pack_func) sec_auth_reply_msg__pack);
 	if (ret < 0) {
@@ -214,11 +221,9 @@ static int check_user_group_status(sec_mod_st * sec, client_entry_st * e,
 					return -1;
 				}
 
-				snprintf(e->username, sizeof(e->username), "%s",
-					 cert_user);
+				strlcpy(e->username, cert_user, sizeof(e->username));
 				if (cert_groups_size > 0 && sec->config->cert_group_oid != NULL && e->groupname[0] == 0)
-					snprintf(e->groupname, sizeof(e->groupname),
-						 "%s", cert_groups[0]);
+					strlcpy(e->groupname, cert_groups[0], sizeof(e->groupname));
 			} else {
 				if (sec->config->cert_user_oid != NULL && cert_user && strcmp(e->username, cert_user) != 0) {
 					seclog(sec, LOG_INFO,
@@ -256,12 +261,12 @@ static int check_user_group_status(sec_mod_st * sec, client_entry_st * e,
  * @result: the auth result
  */
 static
-int handle_sec_auth_res(sec_mod_st * sec, client_entry_st * e, int result)
+int handle_sec_auth_res(int cfd, sec_mod_st * sec, client_entry_st * e, int result)
 {
 	int ret;
 
 	if (result == ERR_AUTH_CONTINUE) {
-		ret = send_sec_auth_reply_msg(sec, e);
+		ret = send_sec_auth_reply_msg(cfd, sec, e);
 		if (ret < 0) {
 			e->status = PS_AUTH_FAILED;
 			seclog(sec, LOG_ERR, "could not send reply auth cmd.");
@@ -271,7 +276,7 @@ int handle_sec_auth_res(sec_mod_st * sec, client_entry_st * e, int result)
 	} else if (result == 0) {
 		e->status = PS_AUTH_COMPLETED;
 
-		ret = send_sec_auth_reply(sec, e, AUTH__REP__OK);
+		ret = send_sec_auth_reply(cfd, sec, e, AUTH__REP__OK);
 		if (ret < 0) {
 			e->status = PS_AUTH_FAILED;
 			seclog(sec, LOG_ERR, "could not send reply auth cmd.");
@@ -279,14 +284,11 @@ int handle_sec_auth_res(sec_mod_st * sec, client_entry_st * e, int result)
 		}
 
 		ret = 0;
-		if (module != NULL && (sec->config->session_control == 0 || module->open_session == NULL)) {
-			del_client_entry(sec, e);
-		} /* else do nothing, and wait for session close/open messages */
 	} else {
 		e->status = PS_AUTH_FAILED;
 		add_ip_to_ban_list(sec, e->ip, time(0) + sec->config->min_reauth_time);
 
-		ret = send_sec_auth_reply(sec, e, AUTH__REP__FAILED);
+		ret = send_sec_auth_reply(cfd, sec, e, AUTH__REP__FAILED);
 		if (ret < 0) {
 			seclog(sec, LOG_ERR, "could not send reply auth cmd.");
 			return ret;
@@ -305,19 +307,102 @@ int handle_sec_auth_res(sec_mod_st * sec, client_entry_st * e, int result)
 
 /* opens or closes a session.
  */
-int handle_sec_auth_session_cmd(sec_mod_st * sec, const SecAuthSessionMsg * req, unsigned cmd)
+int handle_sec_auth_session_cmd(int cfd, sec_mod_st * sec, const SecAuthSessionMsg * req,
+				unsigned cmd)
 {
 	client_entry_st *e;
+	void *lpool;
 	int ret;
 
-	if (module == NULL || module->open_session == NULL)
-		return 0;
-
-	if (sec->config->session_control == 0) {
-		seclog(sec, LOG_ERR, "auth session open/close but session control is disabled!");
-		return 0;
+	if (req->sid.len != SID_SIZE) {
+		seclog(sec, LOG_ERR, "auth session open/close but with illegal sid size (%d)!",
+		       (int)req->sid.len);
+		return -1;
 	}
 
+	e = find_client_entry(sec, req->sid.data);
+	if (e == NULL) {
+		seclog(sec, LOG_INFO, "session open/close but with non-existing SID!");
+		return -1;
+	}
+
+	if (cmd == SM_CMD_AUTH_SESSION_OPEN) {
+		SecAuthSessionReplyMsg rep = SEC_AUTH_SESSION_REPLY_MSG__INIT;
+
+		if (e->time != -1 && time(0) > e->time + sec->config->cookie_timeout) {
+			seclog(sec, LOG_ERR, "session is expired");
+			return -1;
+		}
+
+		if (req->has_cookie == 0 || (req->cookie.len != e->cookie_size) ||
+		    memcmp(req->cookie.data, e->cookie, e->cookie_size) != 0) {
+			seclog(sec, LOG_ERR, "cookie doesn't match the one sent");
+			return -1;
+		}
+
+		if (module != NULL && module->open_session != NULL) {
+			ret = module->open_session(e->auth_ctx, req->sid.data, req->sid.len);
+			if (ret < 0) {
+				e->status = PS_AUTH_FAILED;
+				seclog(sec, LOG_ERR, "could not open session.");
+				rep.reply = AUTH__REP__FAILED;
+			} else {
+				e->have_session = 1;
+				rep.reply = AUTH__REP__OK;
+			}
+		} else {
+			rep.reply = AUTH__REP__OK;
+			e->in_use++;
+		}
+
+		lpool = talloc_new(e);
+		if (lpool == NULL) {
+			return ERR_MEM;
+		}
+
+		if (rep.reply == AUTH__REP__OK && sec->config_module && sec->config_module->get_sup_config) {
+			ret = sec->config_module->get_sup_config(sec->config, e, &rep, lpool);
+			if (ret < 0) {
+				seclog(sec, LOG_ERR, "error reading additional configuration for '%s'", e->username);
+				talloc_free(lpool);
+				return ERR_READ_CONFIG;
+			}
+		}
+
+		ret = send_msg(lpool, cfd, SM_CMD_AUTH_SESSION_REPLY, &rep,
+				(pack_size_func) sec_auth_session_reply_msg__get_packed_size,
+				(pack_func) sec_auth_session_reply_msg__pack);
+		if (ret < 0) {
+			seclog(sec, LOG_WARNING, "sec-mod error in sending session reply");
+		}
+		talloc_free(lpool);
+
+		if (rep.reply != AUTH__REP__OK)
+			del_client_entry(sec, e);
+		else /* set expiration time to unlimited (until someone closes the session) */
+			e->time = -1;
+
+	} else { /* CLOSE */
+		if (req->has_uptime && req->uptime > e->stats.uptime) {
+				e->stats.uptime = req->uptime;
+		}
+		if (req->has_bytes_in && req->bytes_in > e->stats.bytes_in) {
+				e->stats.bytes_in = req->bytes_in;
+		}
+		if (req->has_bytes_out && req->bytes_out > e->stats.bytes_out) {
+				e->stats.bytes_out = req->bytes_out;
+		}
+
+		expire_client_entry(sec, e);
+	}
+
+	return 0;
+}
+
+int handle_sec_auth_stats_cmd(sec_mod_st * sec, const CliStatsMsg * req)
+{
+	client_entry_st *e;
+
 	if (req->sid.len != SID_SIZE) {
 		seclog(sec, LOG_ERR, "auth session open/close but with illegal sid size (%d)!",
 		       (int)req->sid.len);
@@ -330,23 +415,27 @@ int handle_sec_auth_session_cmd(sec_mod_st * sec, const SecAuthSessionMsg * req,
 		return -1;
 	}
 
-	if (cmd == SM_CMD_AUTH_SESSION_OPEN) {
-		ret = module->open_session(e->auth_ctx);
-		if (ret < 0) {
-			e->status = PS_AUTH_FAILED;
-			seclog(sec, LOG_ERR, "could not open session.");
-			del_client_entry(sec, e);
-			return ret;
-		}
-		e->have_session = 1;
-	} else {
-		del_client_entry(sec, e);
+	if (e->status != PS_AUTH_COMPLETED) {
+		seclog(sec, LOG_ERR, "session stats received in unauthenticated client!");
+		return -1;
 	}
 
+	/* stats only increase */
+	if (req->bytes_in > e->stats.bytes_in)
+		e->stats.bytes_in = req->bytes_in;
+	if (req->bytes_out > e->stats.bytes_out)
+		e->stats.bytes_out = req->bytes_out;
+	if (req->uptime > e->stats.uptime)
+		e->stats.uptime = req->uptime;
+
+	if (module == NULL || module->session_stats == NULL)
+		return 0;
+
+	module->session_stats(e->auth_ctx, &e->stats);
 	return 0;
 }
 
-int handle_sec_auth_cont(sec_mod_st * sec, const SecAuthContMsg * req)
+int handle_sec_auth_cont(int cfd, sec_mod_st * sec, const SecAuthContMsg * req)
 {
 	client_entry_st *e;
 	int ret;
@@ -371,15 +460,17 @@ int handle_sec_auth_cont(sec_mod_st * sec, const SecAuthContMsg * req)
 
 	if (e->status != PS_AUTH_INIT) {
 		seclog(sec, LOG_ERR, "auth cont received but we are on state %u!", e->status);
-		return -1;
+		ret = -1;
+		goto cleanup;
 	}
 
 	seclog(sec, LOG_DEBUG, "auth cont for user '%s'", e->username);
 
 	if (req->password == NULL) {
 		seclog(sec, LOG_ERR, "no password given in auth cont for user '%s'",
-		       e->username);
-		return -1;
+			e->username);
+		ret = -1;
+		goto cleanup;
 	}
 
 	ret =
@@ -391,10 +482,11 @@ int handle_sec_auth_cont(sec_mod_st * sec, const SecAuthContMsg * req)
 		       e->username);
 	}
 
-	return handle_sec_auth_res(sec, e, ret);
+ cleanup:
+	return handle_sec_auth_res(cfd, sec, e, ret);
 }
 
-int handle_sec_auth_init(sec_mod_st * sec, const SecAuthInitMsg * req)
+int handle_sec_auth_init(int cfd, sec_mod_st * sec, const SecAuthInitMsg * req)
 {
 	int ret = -1;
 	client_entry_st *e;
@@ -419,7 +511,7 @@ int handle_sec_auth_init(sec_mod_st * sec, const SecAuthInitMsg * req)
 	}
 
 	if (req->hostname != NULL) {
-		snprintf(e->hostname, sizeof(e->hostname), "%s", req->hostname);
+		strlcpy(e->hostname, req->hostname, sizeof(e->hostname));
 	}
 
 	if (sec->config->auth_types & AUTH_TYPE_USERNAME_PASS) {
@@ -428,14 +520,16 @@ int handle_sec_auth_init(sec_mod_st * sec, const SecAuthInitMsg * req)
 		    module->auth_init(&e->auth_ctx, e, req->user_name, req->ip,
 				      sec->config->auth_additional);
 		if (ret < 0) {
-			return ret;
+			goto cleanup;
 		}
 
 		ret =
 		    module->auth_group(e->auth_ctx, req->group_name, e->groupname,
 				       sizeof(e->groupname));
-		if (ret != 0)
-			return -1;
+		if (ret != 0) {
+			ret = -1;
+			goto cleanup;
+		}
 		e->groupname[sizeof(e->groupname) - 1] = 0;
 
 		/* a module is allowed to change the name of the user */
@@ -443,8 +537,7 @@ int handle_sec_auth_init(sec_mod_st * sec, const SecAuthInitMsg * req)
 		    module->auth_user(e->auth_ctx, e->username,
 				      sizeof(e->username));
 		if (ret != 0 && req->user_name != NULL) {
-			snprintf(e->username, sizeof(e->username), "%s",
-				 req->user_name);
+			strlcpy(e->username, req->user_name, sizeof(e->username));
 		}
 	}
 
@@ -454,7 +547,7 @@ int handle_sec_auth_init(sec_mod_st * sec, const SecAuthInitMsg * req)
 
 			for (i=0;i<req->n_cert_group_names;i++) {
 				if (strcmp(req->group_name, req->cert_group_names[i]) == 0) {
-					snprintf(e->groupname, sizeof(e->groupname), "%s", req->cert_group_names[i]);
+					strlcpy(e->groupname, req->cert_group_names[i], sizeof(e->groupname));
 					found = 1;
 					break;
 				}
@@ -463,7 +556,8 @@ int handle_sec_auth_init(sec_mod_st * sec, const SecAuthInitMsg * req)
 			if (found == 0) {
 				seclog(sec, LOG_AUTH, "user '%s' requested group '%s' but is not included on his certificate groups",
 					req->user_name, req->group_name);
-				return -1;
+				ret = -1;
+				goto cleanup;
 			}
 		}
 	}
@@ -488,7 +582,7 @@ int handle_sec_auth_init(sec_mod_st * sec, const SecAuthInitMsg * req)
 
 	ret = 0;
  cleanup:
-	return handle_sec_auth_res(sec, e, ret);
+	return handle_sec_auth_res(cfd, sec, e, ret);
 }
 
 void sec_auth_user_deinit(sec_mod_st * sec, client_entry_st * e)
@@ -499,9 +593,10 @@ void sec_auth_user_deinit(sec_mod_st * sec, client_entry_st * e)
 	seclog(sec, LOG_DEBUG, "auth deinit for user '%s'", e->username);
 	if (e->auth_ctx != NULL) {
 		if (e->have_session) {
-			module->close_session(e->auth_ctx);
+			module->close_session(e->auth_ctx, &e->stats);
 		}
 		module->auth_deinit(e->auth_ctx);
 		e->auth_ctx = NULL;
+		e->have_session = 0;
 	}
 }
