@@ -22,20 +22,24 @@
 # define AUTH_H
 
 #include <main.h>
+#include <sec-mod.h>
 
 #define MAX_AUTH_REQS 8
 #define MAX_GROUPS 32
 
 struct auth_mod_st {
 	unsigned int type;
+	void (*global_init)(void *pool, void* additional);
+	void (*global_deinit)(void);
 	int (*auth_init)(void** ctx, void *pool, const char* username, const char* ip, void* additional);
 	int (*auth_msg)(void* ctx, char* msg, size_t msg_size);
 	int (*auth_pass)(void* ctx, const char* pass, unsigned pass_len);
 	int (*auth_group)(void* ctx, const char *suggested, char *groupname, int groupname_size);
 	int (*auth_user)(void* ctx, char *groupname, int groupname_size);
 
-	int (*open_session)(void *ctx); /* optional, may be null */
-	void (*close_session)(void *ctx); /* optional */
+	int (*open_session)(void *ctx, const void *sid, unsigned sid_size); /* optional, may be null */
+	void (*session_stats)(void *ctx, struct stats_st *stats); /* optional, may be null */
+	void (*close_session)(void *ctx, struct stats_st *stats); /* optional may be null */
 
 	void (*auth_deinit)(void* ctx);
 	void (*group_list)(void *pool, void *additional, char ***groupname, unsigned *groupname_size);
