@@ -19,6 +19,7 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>
  */
 #ifndef SEC_MOD_H
+# define SEC_MOD_H
 
 #include <cookies.h>
 #include <gnutls/abstract.h>
@@ -34,9 +35,14 @@ typedef struct sec_mod_st {
 	struct htable *client_db;
 	struct htable *ban_db;
 
-	int fd;
+	struct config_mod_st *config_module;
 } sec_mod_st;
 
+typedef struct stats_st {
+	uint64_t bytes_in;
+	uint64_t bytes_out;
+	time_t uptime;
+} stats_st;
 
 typedef struct client_entry_st {
 	/* A unique session identifier used to distinguish sessions
@@ -47,8 +53,11 @@ typedef struct client_entry_st {
 	uint8_t sid[SID_SIZE];
 	void * auth_ctx; /* the context of authentication */
 	unsigned have_session; /* whether an auth session is initialized */
+	unsigned in_use; /* counter of users of this structure */
 	unsigned tls_auth_ok;
 
+	stats_st stats;
+
 	unsigned status; /* PS_AUTH_ */
 
 	char ip[MAX_IP_STR]; /* the user's IP */
@@ -60,6 +69,7 @@ typedef struct client_entry_st {
 
 	uint8_t dtls_session_id[GNUTLS_MAX_SESSION_ID];
 
+	/* The time this client entry was last modified (created or closed) */
 	time_t time;
 } client_entry_st;
 
@@ -69,25 +79,31 @@ unsigned sec_mod_client_db_elems(sec_mod_st *sec);
 client_entry_st * new_client_entry(sec_mod_st *sec, const char *ip);
 client_entry_st * find_client_entry(sec_mod_st *sec, uint8_t sid[SID_SIZE]);
 void del_client_entry(sec_mod_st *sec, client_entry_st * e);
+void expire_client_entry(sec_mod_st *sec, client_entry_st * e);
 void cleanup_client_entries(sec_mod_st *sec);
 
 #ifdef __GNUC__
 # define seclog(sec, prio, fmt, ...) \
-	if (prio != LOG_DEBUG || sec->config->debug != 0) { \
+	if (prio != LOG_DEBUG || sec->config->debug >= 3) { \
 		syslog(prio, "sec-mod: "fmt, ##__VA_ARGS__); \
 	}
 #else
 # define seclog(sec,prio,...) \
-	if (prio != LOG_DEBUG || sec->config->debug != 0) { \
+	if (prio != LOG_DEBUG || sec->config->debug >= 3) { \
 		 syslog(prio, __VA_ARGS__); \
 	}
 #endif
 
-void sec_auth_init(struct cfg_st *config);
+void  seclog_hex(const struct sec_mod_st* sec, int priority,
+		const char *prefix, uint8_t* bin, unsigned bin_size, unsigned b64);
 
-int handle_sec_auth_init(sec_mod_st *sec, const SecAuthInitMsg * req);
-int handle_sec_auth_cont(sec_mod_st *sec, const SecAuthContMsg * req);
-int handle_sec_auth_session_cmd(sec_mod_st * sec, const SecAuthSessionMsg * req, unsigned cmd);
+void sec_auth_init(sec_mod_st *sec, struct cfg_st *config);
+void sec_auth_reinit(sec_mod_st *sec, struct cfg_st *config);
+
+int handle_sec_auth_init(int cfd, sec_mod_st *sec, const SecAuthInitMsg * req);
+int handle_sec_auth_cont(int cfd, sec_mod_st *sec, const SecAuthContMsg * req);
+int handle_sec_auth_session_cmd(int cfd, sec_mod_st *sec, const SecAuthSessionMsg *req, unsigned cmd);
+int handle_sec_auth_stats_cmd(sec_mod_st * sec, const CliStatsMsg * req);
 void sec_auth_user_deinit(sec_mod_st * sec, client_entry_st * e);
 
 void sec_mod_server(void *main_pool, struct cfg_st *config, const char *socket_file,
