@@ -36,10 +36,17 @@ typedef void (*sighandler_t)(int);
 # define SIGHANDLER_T sighandler_t
 #endif
 
+#if defined(__linux__) && defined(ENABLE_LINUX_NS)
+pid_t safe_fork(void);
+#else
+# define safe_fork fork
+#endif
+
+void pr_set_undumpable(const char* mod);
 void kill_on_parent_kill(int sig);
 
 SIGHANDLER_T ocsignal(int signum, SIGHANDLER_T handler);
 
-int check_upeer_id(const char *mod, int cfg, uid_t uid, uid_t gid, uid_t *ruid);
+int check_upeer_id(const char *mod, int debug, int cfg, uid_t uid, uid_t gid, uid_t *ruid);
 
 #endif
